@@ -23,7 +23,7 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-# This is v1.0 of this boilerplate.
+# This is v1.1 of this boilerplate.
 
 
 import sys
@@ -64,7 +64,7 @@ class ModuleConfiguration(object):
 
     # The version of the module as a string.  Set it to None if you don't
     # provide version information.
-    version = '2.8.2'
+    version = '2.8.3'
 
     # Set if a configuration script is provided that handles versions of PyQt4
     # prior to v4.10 (i.e. versions where the pyqtconfig.py module is
@@ -94,8 +94,8 @@ class ModuleConfiguration(object):
     pyqt5_is_default = False
 
     # The name (without the .api extension) of the name of the QScintilla API
-    # file to be generated.  If it is None or empty then an API file is not
-    # generated.
+    # file to be generated.  If it is None or an empty string then an API file
+    # is not generated.
     qscintilla_api_file = 'QScintilla2'
 
     # The email address that will be included when an error in the script is
@@ -244,6 +244,21 @@ class ModuleConfiguration(object):
             inform("The QScintilla .sip files will be installed in %s." %
                     target_configuration.qsci_sip_dir)
 
+    def pre_code_generation(self, target_config):
+        """ Perform any module specific initialisation prior to generating the
+        code.  target_config is the target configuration.
+        """
+
+        # Nothing to do.
+
+    def get_sip_flags(self, target_configuration):
+        """ Return the list of module-specific flags to pass to SIP.
+        target_configuration is the target configuration.
+        """
+
+        # Nothing to do.
+        return []
+
     def get_sip_file(self, target_configuration):
         """ Return the name of the module's .sip file.  target_configuration is
         the target configuration.
@@ -269,17 +284,18 @@ class ModuleConfiguration(object):
 
     def get_qmake_configuration(self, target_configuration):
         """ Return a dict of qmake configuration values for CONFIG, DEFINES,
-        INCLUDEPATH and LIBS.  target_configuration is the target
-        configuration.
+        INCLUDEPATH, LIBS and QT.  If value names (i.e. dict keys) have either
+        'Qt4' or 'Qt5' prefixes then they are specific to the corresponding
+        version of Qt.  target_configuration is the target configuration.
         """
 
-        qmake = dict(CONFIG='qscintilla2')
+        qmake = {'CONFIG': 'qscintilla2'}
 
         if target_configuration.qsci_inc_dir is not None:
-            qmake['INCLUDEPATH'] = quote(target_config.qsci_inc_dir)
+            qmake['INCLUDEPATH'] = quote(target_configuration.qsci_inc_dir)
 
         if target_configuration.qsci_lib_dir is not None:
-            qmake['LIBS'] = '-L%s' % quote(target_config.qsci_inc_dir)
+            qmake['LIBS'] = '-L%s' % quote(target_configuration.qsci_lib_dir)
 
         if target_configuration.qsci_is_dll:
             qmake['DEFINES'] = 'QSCINTILLA_DLL'
@@ -288,7 +304,9 @@ class ModuleConfiguration(object):
 
     def get_mac_wrapped_library_file(self, target_configuration):
         """ Return the full pathname of the file that implements the library
-        being wrapped by the module as it would be called on OS/X.
+        being wrapped by the module as it would be called on OS/X so that the
+        module will reference it explicitly without DYLD_LIBRARY_PATH being
+        set.  If it is None or an empty string then the default is used.
         target_configuration is the target configuration.
         """
 
@@ -382,6 +400,38 @@ def read_define(filename, define):
     return value
 
 
+def version_from_string(version_str):
+    """ Convert a version string of the form m, m.n or m.n.o to an encoded
+    version number (or None if it was an invalid format).  version_str is the
+    version string.
+    """
+
+    parts = version_str.split('.')
+    if not isinstance(parts, list):
+        return None
+
+    if len(parts) == 1:
+        parts.append('0')
+
+    if len(parts) == 2:
+        parts.append('0')
+
+    if len(parts) != 3:
+        return None
+
+    version = 0
+
+    for part in parts:
+        try:
+            v = int(part)
+        except ValueError:
+            return None
+
+        version = (version << 8) + v
+
+    return version
+
+
 def _format(msg, left_margin=0, right_margin=78):
     """ Format a message by inserting line breaks at appropriate places.  msg
     is the text of the message.  left_margin is the position of the left
@@ -459,7 +509,7 @@ class _ConfigurationFileParser:
                 last_name = None
                 continue
 
-            parts = l.split('=', maxsplit=1)
+            parts = l.split('=', 1)
             if len(parts) == 2:
                 name = parts[0].strip()
                 value = parts[1].strip()
@@ -504,11 +554,11 @@ class _ConfigurationFileParser:
                 return default
 
         # Handle any extrapolations.
-        parts = value.split('%(', maxsplit=1)
+        parts = value.split('%(', 1)
         while len(parts) == 2:
             prefix, tail = parts
 
-            parts = tail.split(')', maxsplit=1)
+            parts = tail.split(')', 1)
             if len(parts) != 2:
                 error(
                         "Configuration file contains unterminated "
@@ -527,7 +577,7 @@ class _ConfigurationFileParser:
 
             value = prefix + xtra_value + suffix
 
-            parts = value.split('%(', maxsplit=1)
+            parts = value.split('%(', 1)
 
         return value
 
@@ -712,7 +762,7 @@ class _TargetConfiguration:
                 if len(parts) != 2 or parts[0] != 'PyQt':
                     continue
 
-                section_pyqt_version = _version_from_string(parts[1])
+                section_pyqt_version = version_from_string(parts[1])
                 if section_pyqt_version is None:
                     continue
 
@@ -751,7 +801,7 @@ class _TargetConfiguration:
             flags.append('-t')
             flags.append(self._get_platform_tag())
 
-            qt_version = _version_from_string(self.qt_version_str)
+            qt_version = version_from_string(self.qt_version_str)
             if qt_version is None:
                 error("Unable to determine the version of Qt.")
 
@@ -924,7 +974,11 @@ class _TargetConfiguration:
             self.sysroot = opts.sysroot
 
         if self.pyqt_package is not None:
-            self.pyqt_package = opts.pyqt_package
+            try:
+                self.pyqt_package = opts.pyqt_package
+            except AttributeError:
+                # Multiple PyQt versions are not supported.
+                pass
 
             self.module_dir = os.path.join(self.py_module_dir,
                     self.pyqt_package)
@@ -1165,6 +1219,9 @@ def _generate_code(target_config, opts, module_config):
     # Build the SIP command line.
     argv = [quote(target_config.sip)]
 
+    # Add the module-specific flags.
+    argv.extend(module_config.get_sip_flags(target_config))
+
     if target_config.pyqt_package is not None:
         # Get the flags used for the main PyQt module.
         argv.extend(target_config.pyqt_sip_flags.split())
@@ -1214,6 +1271,29 @@ def _generate_code(target_config, opts, module_config):
     _run_qmake(target_config, opts.verbose, pro, module_config)
 
 
+def _get_qt_qmake_config(qmake_config, qt_version):
+    """ Return a dict of qmake configuration values for a specific Qt version.
+    """
+
+    qt_qmake_config = {}
+
+    for name, value in qmake_config.items():
+        name_parts = name.split(':')
+        if len(name_parts) == 2 and name_parts[0] == qt_version:
+            qt_qmake_config[name_parts[1]] = value
+
+    return qt_qmake_config
+
+
+def _write_qt_qmake_config(qt_qmake_config, pro):
+    """ Write the qmake configuration values to a .pro file. """
+
+    for name in ('QT', 'CONFIG', 'DEFINES', 'INCLUDEPATH', 'LIBS'):
+        value = qt_qmake_config.get(name)
+        if value:
+            pro.write('    %s += %s\n' % (name, value))
+
+
 def _generate_pro(target_config, opts, module_config):
     """ Generate the .pro file for the module and return its name.
     target_config is the target configuration.  opts are the command line
@@ -1232,6 +1312,11 @@ def _generate_pro(target_config, opts, module_config):
     pro = open(pro_name, 'w')
 
     pro.write('TEMPLATE = lib\n')
+
+    qt = qmake_config.get('QT')
+    if qt:
+        pro.write('QT += %s\n' % qt)
+
     pro.write('CONFIG += %s\n' % ('debug' if opts.debug else 'release'))
     pro.write('CONFIG += %s\n' % ('staticlib' if opts.static else 'plugin'))
 
@@ -1239,6 +1324,23 @@ def _generate_pro(target_config, opts, module_config):
     if config:
         pro.write('CONFIG += %s\n' % config)
 
+    qt5_qmake_config = _get_qt_qmake_config(qmake_config, 'Qt5')
+    qt4_qmake_config = _get_qt_qmake_config(qmake_config, 'Qt4')
+
+    if qt5_qmake_config or qt4_qmake_config:
+        pro.write('''
+greaterThan(QT_MAJOR_VERSION, 4) {
+''')
+
+        if qt5_qmake_config:
+            _write_qt_qmake_config(qt5_qmake_config, pro)
+
+        if qt4_qmake_config:
+            pro.write('} else {\n')
+            _write_qt_qmake_config(qt4_qmake_config, pro)
+
+        pro.write('}\n')
+
     mname = module_config.name
 
     if not opts.static:
@@ -1323,8 +1425,6 @@ INSTALLS += sip
         pro.write('LIBS += %s\n' % libs)
 
     if not opts.static:
-        dylib = module_config.get_mac_wrapped_library_file(target_config)
-
         pro.write('''
 win32 {
     QMAKE_POST_LINK = $(COPY_FILE) $(DESTDIR_TARGET) $$PY_MODULE
@@ -1337,11 +1437,17 @@ macx {
     greaterThan(QT_MAJOR_VERSION, 4) {
         QMAKE_LFLAGS += "-install_name $$absolute_path($$PY_MODULE, $$target.path)"
     }
+''')
 
+        dylib = module_config.get_mac_wrapped_library_file(target_config)
+
+        if dylib:
+            pro.write('''
     QMAKE_POST_LINK = $$QMAKE_POST_LINK$$escape_expand(\\\\n\\\\t)$$quote(install_name_tool -change %s %s $$PY_MODULE)
-}
 ''' % (os.path.basename(dylib), dylib))
 
+        pro.write('}\n')
+
     pro.write('\n')
     pro.write('TARGET = %s\n' % mname)
     pro.write('HEADERS = sipAPI%s.h\n' % mname)
@@ -1476,7 +1582,7 @@ def _check_sip(target_config, module_config):
     pipe.close()
 
     if 'snapshot' not in version_str and 'preview' not in version_str:
-        version = _version_from_string(version_str)
+        version = version_from_string(version_str)
         if version is None:
             error(
                     "'%s -V' generated unexpected output: '%s'." % (
@@ -1484,7 +1590,7 @@ def _check_sip(target_config, module_config):
 
         min_sip_version = module_config.minimum_sip_version
         if min_sip_version:
-            min_version = _version_from_string(min_sip_version)
+            min_version = version_from_string(min_sip_version)
             if version < min_version:
                 error(
                         "This version of %s requires sip %s or later." %
@@ -1494,38 +1600,6 @@ def _check_sip(target_config, module_config):
     target_config.sip_version = version_str
 
 
-def _version_from_string(version_str):
-    """ Convert a version string of the form m, m.n or m.n.o to an encoded
-    version number (or None if it was an invalid format).  version_str is the
-    version string.
-    """
-
-    parts = version_str.split('.')
-    if not isinstance(parts, list):
-        return None
-
-    if len(parts) == 1:
-        parts.append('0')
-
-    if len(parts) == 2:
-        parts.append('0')
-
-    if len(parts) != 3:
-        return None
-
-    version = 0
-
-    for part in parts:
-        try:
-            v = int(part)
-        except ValueError:
-            return None
-
-        version = (version << 8) + v
-
-    return version
-
-
 def _main(argv, module_config):
     """ Create the configuration module module.  argv is the list of command
     line arguments.  module_config is the module configuration.
@@ -1574,6 +1648,9 @@ def _main(argv, module_config):
     # Tell the user what's been found.
     _inform_user(target_config, module_config)
 
+    # Allow for module specific hacks.
+    module_config.pre_code_generation(target_config)
+
     # Generate the code.
     _generate_code(target_config, opts, module_config)
 
